package com.ydl.user

import com.ydl.user.http.request.LoginParam
import com.ydl.user.route.UserIn
import com.yidianling.ydlcommon.UserInfoCache
import com.yidianling.ydlcommon.http.RxUtils
import com.yidianling.ydlcommon.http.ThrowableConsumer
import com.yidianling.router.im.IMLoginInfo
import com.yidianling.router.im.IMRequestCallback
import com.yidianling.router.user.UserResponse
import com.yidianling.ydlcommon.event.RefreshEvent
import com.yidianling.ydlcommon.log.LogHelper
import com.yidianling.ydlcommon.mvp.RxPresenter
import de.greenrobot.event.EventBus
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.functions.Consumer

/**
 * author : Zhangwenchao
 * e-mail : zhangwch@yidianling.com
 * time   : 2018/02/02
 */
class LoginPresenter(view: LoginContract.View): RxPresenter<LoginContract.View, LoginContract.Model>(view), LoginContract.Presenter {

    override fun createModel(): LoginContract.Model {
        return LoginModel()
    }

    override fun login(param: LoginParam) {
        val d = model.login(param).compose(RxUtils.resultData())
                .doOnNext {
                    //存储登录缓存
                    LoginHelper.login(it)
                }
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe { view.startLogin() }
                .doAfterTerminate {
                    view.onLoginStop()
                }
                .subscribe(Consumer {
                    LogHelper.getInstance().writeLogSync("登录成功")
                    EventBus.getDefault().post(RefreshEvent(1))
                    view.loginSuccess(it)
                    val info = IMLoginInfo(it.uid.toString(), it.hxpwd!!)
                    val callback = object : IMRequestCallback<IMLoginInfo> {
                        override fun onSuccess(t: IMLoginInfo?) {
                            setHXInfo(it)
                        }

                        override fun onFailed(i: Int) {
                            view.loginFail("账号登录异常")
                        }

                        override fun onException(throwable: Throwable?) {
                            view.loginFail("未知错误02")
                        }
                    }
                    UserIn.imSetAccount(info.account)
                    UserIn.imLogin(info, callback)
                }, object : ThrowableConsumer() {
                    override fun accept(msg: String) {
                        LogHelper.getInstance().writeLogSync("登录失败：$msg")
                        view.loginFail(msg)
                    }
                })


        addDisposable(d)
    }

    private fun setHXInfo(userInfo: UserResponse?) {
        try {
            if (userInfo != null) {
                UserIn.imSetAccount(userInfo.uid.toString())
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
}