package com.ydl.user.ui.login.presenter

import android.text.TextUtils
import com.yidianling.common.tools.ToastUtil
import com.yidianling.router.user.UserResponse
import com.ydl.user.LoginUtils
import com.ydl.user.UserHelper
import com.ydl.user.http.request.PhoneLoginPwdParam
import com.ydl.user.http.request.PhoneResetPwdParam
import com.ydl.user.ui.login.contract.IInputPassWordContract
import com.ydl.user.ui.login.model.InputPassWordModelImpl
import com.yidianling.ydlcommon.http.YdlRetrofitUtils
import com.yidianling.ydlcommon.log.LogHelper
import com.yidianling.ydlcommon.mvp.RxPresenter
import com.yidianling.ydlcommon.router.YdlCommonOut
import com.yidianling.ydlcommon.tool.StringUtils
import com.yidianling.ydlcommon.utils.actionutil.ActionCountUtils
import com.yidianling.ydlcommon.utils.UMEventUtils
import com.yidianling.ydlcommon.utils.YDLCacheUtils
import com.yidianling.ydlcommon.utils.actionutil.BIConstants
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.math.BigDecimal

/**
 * @author jiucheng
 * @描述:密码输入页面presenter
 * @Copyright Copyright (c) 2018
 * @Company 壹点灵
 * @date 2018/12/2
 */
class InputPassWordPresenterImpl(view: IInputPassWordContract.View) : RxPresenter<IInputPassWordContract.View, IInputPassWordContract.Model>(view), IInputPassWordContract.Presenter {

    override fun createModel(): IInputPassWordContract.Model {
        return InputPassWordModelImpl()
    }

    /**
     * 重新设置密码
     */
    override fun resetPwd(newPassword: String, phone: String, msgCode: String, phoneCountryCode: String) {

        ActionCountUtils.count(UserHelper.getUserInfo()!!.uid, BIConstants.PART_ID_LOGIN_MAIN,
                BIConstants.POSITION_LOGIN_CLICK, YdlRetrofitUtils.SERVER_API_JAVA_URL,
                YdlRetrofitUtils.SERVER_API_JAVA_URL + "user/reset_pwd", BIConstants.ACTION_TYPE_RESET_PWD)

        var param = PhoneResetPwdParam(StringUtils.md5(newPassword), phoneCountryCode, msgCode, phone)
        model.resetPwd(param)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe { view.showLoading(true) }
                .doAfterTerminate { view.showLoading(false) }
                .subscribe({
                    if (it?.data == null) {
                        ToastUtil.toastShort(it.msg)
                    } else {
                        if (it.data.userInfo?.user_type == 2) {
                            view.showNormalDialog()
                        } else {
                            ToastUtil.toastShort("修改密码成功")
                            saveUserData(it.data)
                            ActionCountUtils.count(UserHelper.getUserInfo()!!.uid, BIConstants.PART_ID_LOGIN_MAIN,
                                    BIConstants.POSITION_LOGIN_SUCCESS_CLICK, YdlRetrofitUtils.SERVER_API_JAVA_URL,
                                    YdlRetrofitUtils.SERVER_API_JAVA_URL + "user/reset_pwd", BIConstants.ACTION_TYPE_RESET_PWD)
                            if (it.data.firstLogin == 1) {//第一次登录：是注册
                                // 重设密码登录已经不可能是首次登录，所以不进行百度埋点
                                //view.baiduActionBury()
                                UMEventUtils.loginSuccess("InputPassWordActivity|Reg_succ")
                            } else {
                                UMEventUtils.loginSuccess("InputPassWordActivity|Login_succ")
                            }
                            view.closeActivity()
                        }
                    }
                }, {
                    YdlCommonOut.showToast(it.message!!)
                })
    }


    /**
     * 手机号密码登录
     */
    override fun userLoginByPassword(inputPassword: String, phone: String, phoneCountryCode: String) {

        ActionCountUtils.count(UserHelper.getUserInfo()!!.uid, BIConstants.PART_ID_LOGIN_MAIN,
                BIConstants.POSITION_LOGIN_CLICK, YdlRetrofitUtils.SERVER_API_JAVA_URL,
                YdlRetrofitUtils.SERVER_API_JAVA_URL + "user/login_pwd", BIConstants.ACTION_TYPE_PWD_LOGIN)

        var param = PhoneLoginPwdParam(StringUtils.md5(inputPassword), phoneCountryCode, phone)
        model.userLoginByPassword(param)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe { view.showLoading(true) }
                .doAfterTerminate { view.showLoading(false) }
                .subscribe({
                    if (it?.data == null) {
                        ToastUtil.toastShort(it.msg)
                        view.startAnim()
                    } else {
                        if (it.data.userInfo?.user_type == 2) {
                            view.showNormalDialog()
                        } else {
                            saveUserData(it.data)
                            if (it.data.firstLogin == 1) {//第一次登录：是注册
                                view.baiduActionBury()
                                UMEventUtils.loginSuccess("InputPassWordActivity|Reg_succ")
                            } else {
                                ActionCountUtils.count(UserHelper.getUserInfo()!!.uid, BIConstants.PART_ID_LOGIN_MAIN,
                                        BIConstants.POSITION_LOGIN_SUCCESS_CLICK, YdlRetrofitUtils.SERVER_API_JAVA_URL,
                                        YdlRetrofitUtils.SERVER_API_JAVA_URL + "user/login_pwd", BIConstants.ACTION_TYPE_PWD_LOGIN)
                                UMEventUtils.loginSuccess("InputPassWordActivity|Login_succ")
                            }
                            view.closeActivity()
                        }
                    }
                }, {
                    YdlCommonOut.showToast(it.message!!)
                })
    }

    /**
     * 保存登录信息
     */
    private fun saveUserData(userResponse: UserResponse) {
        if (!TextUtils.isEmpty(userResponse.userInfo!!.phone)) {
            YDLCacheUtils.saveUserPhone(userResponse.userInfo!!.phone)
            YDLCacheUtils.saveCountryPhoneCode(userResponse.userInfo!!.country_code)
        }
        if (userResponse.uid!!.contains(".")) {
            val bd = BigDecimal(userResponse.uid)
            var uid = bd.toPlainString()
            if (uid.endsWith(".0")) {
                uid = uid.replace(".0", "")
            }
            userResponse.userInfo!!.uid = uid
            userResponse.uid = uid
        }
        LoginUtils.saveData(userResponse)
        LogHelper.getInstance().writeLogSync("登录成功")
        LoginUtils.onLogin(userResponse)
    }
}
