package com.ydl.user.ui.login.presenter

import android.app.Activity
import android.text.TextUtils
import com.umeng.socialize.bean.SHARE_MEDIA
import com.yidianling.common.tools.ToastUtil
import com.yidianling.router.user.UserResponse
import com.ydl.user.LoginUtils
import com.ydl.user.UserHelper
import com.ydl.user.http.UserHttpImpl
import com.ydl.user.http.request.BindPhoneJavaParam
import com.ydl.user.http.request.CheckAliAuthParam
import com.ydl.user.http.request.PhoneLoginAutoParam
import com.ydl.user.ui.login.contract.ILoginContract
import com.ydl.user.ui.login.model.LoginModelImpl
import com.yidianling.ydlcommon.http.YdlRetrofitUtils
import com.yidianling.ydlcommon.log.LogHelper
import com.yidianling.ydlcommon.mvp.RxPresenter
import com.yidianling.ydlcommon.router.YdlCommonOut
import com.yidianling.ydlcommon.utils.actionutil.ActionCountUtils
import com.yidianling.ydlcommon.utils.UMEventUtils
import com.yidianling.ydlcommon.utils.YDLCacheUtils
import com.yidianling.ydlcommon.utils.actionutil.BIConstants
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.math.BigDecimal

/**
 * @author jiucheng
 * @描述:
 * @Copyright Copyright (c) 2018
 * @Company 壹点灵
 * @date 2018/11/29
 */
class LoginPresenterImpl(view: ILoginContract.View) : RxPresenter<ILoginContract.View, ILoginContract.Model>(view), ILoginContract.Presenter {
    override fun bindPhone(param: BindPhoneJavaParam) {
        model.bindPhone(param)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe { view.showLoading(true) }
                .doAfterTerminate { view.showLoading(false) }
                .subscribe({
                    if (it.code == 200) {
                        ToastUtil.toastShort("手机号绑定成功")
                        view.bindSuccess()
                    } else {
                        ToastUtil.toastShort(it.msg)
                    }
                }, {
                    ToastUtil.toastShort(it.message)
                })

    }

    /**
     * 通过一键认证服务登陆
     */
    override fun autoLogin(bean: PhoneLoginAutoParam) {

        ActionCountUtils.count(UserHelper.getUserInfo()!!.uid, BIConstants.PART_ID_LOGIN_MAIN,
                BIConstants.POSITION_LOGIN_CLICK, YdlRetrofitUtils.SERVER_API_JAVA_URL,
                YdlRetrofitUtils.SERVER_API_JAVA_URL + "user/login_direct", BIConstants.ACTION_TYPE_DIRECT_LOGIN)

        model.autoLogin(bean)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe { view.showLoading(true) }
                .doAfterTerminate { view.showLoading(false) }
                .subscribe({
                    if (it.code == 200) {
                        //绑定手机号
                        saveUserData(it.data)
                        if (it.data.firstLogin == 1) {//第一次登录：是注册
                            view.baiduActionBury(BIConstants.POSITION_ALIYUN_REGISTER_CLICK, YdlRetrofitUtils.SERVER_API_JAVA_URL,
                                    YdlRetrofitUtils.SERVER_API_JAVA_URL + "user/login_direct")
                            UMEventUtils.loginSuccess("RegisterAndLoginActivity|Reg_succ")
                        } else {
                            ActionCountUtils.count(UserHelper.getUserInfo()!!.uid, BIConstants.PART_ID_LOGIN_MAIN,
                                    BIConstants.POSITION_LOGIN_SUCCESS_CLICK, YdlRetrofitUtils.SERVER_API_JAVA_URL,
                                    YdlRetrofitUtils.SERVER_API_JAVA_URL + "user/login_direct", BIConstants.ACTION_TYPE_DIRECT_LOGIN)
                            UMEventUtils.loginSuccess("RegisterAndLoginActivity|Login_succ")
                        }
                        view.autoLoginSuccess()
                    } else {
//                        ToastUtil.toastShort(it.msg)
                        view.aliAuthFailed()
                    }
                }, {
                    //走验证码、密码登录
                    view.aliAuthFailed()
                })
    }


    /**
     * 保存登录信息
     */
    private fun saveUserData(userResponse: UserResponse) {
        if (!TextUtils.isEmpty(userResponse.userInfo!!.phone)) {
            YDLCacheUtils.saveUserPhone(userResponse.userInfo!!.phone)
            YDLCacheUtils.saveCountryPhoneCode(userResponse.userInfo!!.country_code)
        }
        if (userResponse.uid!!.contains(".")) {
            val bd = BigDecimal(userResponse.uid)
            var uid = bd.toPlainString()
            if (uid.endsWith(".0")) {
                uid = uid.replace(".0", "")
            }
            userResponse.userInfo!!.uid = uid
            userResponse.uid = uid
        }
        LoginUtils.saveData(userResponse)
        LogHelper.getInstance().writeLogSync("登录成功")
        LoginUtils.onLogin(userResponse)
    }

    /**
     * 服务端校验阿里一键认证
     */
    override fun checkAliAuth(param: CheckAliAuthParam) {
        model.checkAliAuth(param)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({
                    if (it?.data == null) {
                        view.aliAuthFailed()
                    } else {
                        view.getAuthToken(it.data.vendorConfigDTOs)
                    }
                }, {
                    view.aliAuthFailed()
                })
    }

    /**
     * 校验手机号：是否是用户版号码、是否有设置密码、是否被绑定
     *@param isBind 是否是第三方登录成功后的绑定操作
     */
    override fun checkPhoneStatus(phone: String, countryCode: String, isBind: Boolean) {
        view.showLoading(true)
        model.checkPhoneStatus(phone, countryCode)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({
                    if (it?.data == null || it?.code != 200) {
                        ToastUtil.toastShort(it.msg)
                        view.showLoading(false)
                        return@subscribe
                    }
                    if (isBind) {//第三方登录成功后的绑定逻辑
                        if (it.data.isRegistered == 1) {
                            view.showLoading(false)
                            view.showHasBindedDialog()
                        } else {
                            view.checkPhoneStatusSuccess(it.data, phone)
                        }
                    } else {//登录逻辑
                        if (it.data.isDoctor == 1) {//是专家账号
                            view.showLoading(false)
                            view.showNormalDialog()
                        } else {
                            view.checkPhoneStatusSuccess(it.data, phone)
                        }
                    }
                }, {
                    view.showLoading(false)
                    if (null != it && !TextUtils.isEmpty(it.message)){
                        YdlCommonOut.showToast(it.message!!)
                    }
                })
    }

    /**
     * 三方登录
     */
    override fun loginByThird(activity: Activity, media: SHARE_MEDIA) {

        val sign1 = if (media == SHARE_MEDIA.QQ) BIConstants.ACTION_TYPE_QQ_THIRD_LOGIN else BIConstants.ACTION_TYPE_WX_THIRD_LOGIN
        ActionCountUtils.count(UserHelper.getUserInfo()!!.uid, BIConstants.PART_ID_LOGIN_MAIN,
                BIConstants.POSITION_LOGIN_CLICK, "友盟三方登录url无法获取",
                "友盟三方登录api无法获取", sign1)

        UserHttpImpl.getInstance()
                .umLogin(activity, media)
                .doOnSubscribe { view.showProgressView() }
                .doAfterTerminate {
                    view.dismissProgressView()
                }
                .observeOn(Schedulers.io())
                .flatMap { thirdLoginParam -> UserHttpImpl.getInstance().thirdPartJavaLogin(thirdLoginParam) }
                .doOnNext { LoginUtils.saveData(it.data) }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({
                    view.dismissProgressView()
                    if (it?.data == null) {
                        ToastUtil.toastShort(it.msg)
                        return@subscribe
                    }
                    LoginUtils.onLogin(it.data)
                    view.thirdLoginSuccess(it.data.userInfo!!)
                    LogHelper.getInstance().writeLogSync(media.getName() + "登录成功")
                    if (media == SHARE_MEDIA.QQ) {
                        if (it.data.firstLogin == 1) {//第一次登录：是注册
                            view.baiduActionBury(BIConstants.POSITION_QQ_REGISTER_CLICK, "友盟三方登录url无法获取", "友盟三方登录api无法获取")
                            UMEventUtils.loginSuccess("RegisterAndLoginActivity|Reg_succ_byQQ")
                        } else {
                            ActionCountUtils.count(UserHelper.getUserInfo()!!.uid, BIConstants.PART_ID_LOGIN_MAIN,
                                    BIConstants.POSITION_LOGIN_SUCCESS_CLICK, "友盟三方登录url无法获取",
                                    "友盟三方登录api无法获取", sign1)
                            UMEventUtils.loginSuccess("RegisterAndLoginActivity|Login_succ_byQQ")
                        }
                    } else {
                        if (it.data.firstLogin == 1) {//第一次登录：是注册
                            view.baiduActionBury(BIConstants.POSITION_WX_REGISTER_CLICK, "友盟三方登录url无法获取", "友盟三方登录api无法获取")
                            UMEventUtils.loginSuccess("RegisterAndLoginActivity|Reg_succ_byWechat")
                        } else {
                            ActionCountUtils.count(UserHelper.getUserInfo()!!.uid, BIConstants.PART_ID_LOGIN_MAIN,
                                    BIConstants.POSITION_LOGIN_SUCCESS_CLICK, "友盟三方登录url无法获取",
                                    "友盟三方登录api无法获取", sign1)
                            UMEventUtils.loginSuccess("RegisterAndLoginActivity|Login_succ_byWechat")
                        }
                    }
                }, {
                    view.dismissProgressView()
                    YdlCommonOut.showToast(it.message!!)
                })
    }

    override fun createModel(): ILoginContract.Model {
        return LoginModelImpl()
    }
}