package com.ydl.user.ui.login.presenter

import android.text.TextUtils
import com.yidianling.common.tools.ToastUtil
import com.yidianling.router.user.UserResponse
import com.ydl.user.LoginUtils
import com.ydl.user.UserHelper
import com.ydl.user.http.request.BindPhoneJavaParam
import com.ydl.user.http.request.PhoneLoginCodeParam
import com.ydl.user.ui.login.contract.IVerificationCodeContract
import com.ydl.user.ui.login.model.VerificationCodeModelImpl
import com.yidianling.ydlcommon.http.YdlRetrofitUtils
import com.yidianling.ydlcommon.log.LogHelper
import com.yidianling.ydlcommon.mvp.RxPresenter
import com.yidianling.ydlcommon.utils.actionutil.ActionCountUtils
import com.yidianling.ydlcommon.utils.UMEventUtils
import com.yidianling.ydlcommon.utils.YDLCacheUtils
import com.yidianling.ydlcommon.utils.actionutil.BIConstants
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.math.BigDecimal

/**
 * @author jiucheng
 * @描述:
 * @Copyright Copyright (c) 2018
 * @Company 壹点灵
 * @date 2018/12/3
 */
class VerificationCodePresenterImpl(view: IVerificationCodeContract.View) : RxPresenter<IVerificationCodeContract.View, IVerificationCodeContract.Model>(view), IVerificationCodeContract.Presenter {

    override fun createModel(): IVerificationCodeContract.Model {
        return VerificationCodeModelImpl()
    }

    //发送登录验证码
    override fun sendLoginCode(phone: String, phoneCountryCode: String) {
        model.sendLoginMsgCode(phone, phoneCountryCode)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe { view.showLoading(true) }
                .doAfterTerminate { view.showLoading(false) }
                .subscribe({
                    if (it.code == 200) {
                        YDLCacheUtils.saveCodeTime(phone, System.currentTimeMillis().toString())
//                        ToastUtil.toastShort("验证码已发送")
                        view.startCountdown(60)
                    } else {
                        view.startAnim()
                    }
                }, {
                    view.startAnim()
                    ToastUtil.toastShort(it.message)
                })
    }

    /**
     * 重置密码的验证码
     */
    override fun sendResetCode(phone: String, countryCode: String) {
        model.sendResetCode(phone, countryCode)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe { view.showLoading(true) }
                .doAfterTerminate { view.showLoading(false) }
                .subscribe({
                    if (it.code == 200) {
                        YDLCacheUtils.saveResetCodeTime(phone, System.currentTimeMillis().toString())
//                        ToastUtil.toastShort("验证码已发送")
                        view.startCountdown(60)
                    } else {
                        view.startAnim()
                    }
                }, {
                    view.startAnim()
                    ToastUtil.toastShort(it.message)
                })
    }

    /**
     * 绑定手机号
     */
    override fun bindPhone(param: BindPhoneJavaParam) {
        model.bindPhone(param)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe { view.showLoading(true) }
                .doAfterTerminate { view.showLoading(false) }
                .subscribe({
                    if (it.code == 200) {
                        //绑定手机号成功后清空本地保存时间
                        YDLCacheUtils.saveCodeTime(param.phoneNumber, "")

                        ToastUtil.toastShort("手机号绑定成功")
                        view.bindSuccess()
                    } else {
                        view.startAnim()
                    }
                }, {
                    view.startAnim()
                    ToastUtil.toastShort(it.message)
                })

    }

    /**
     * 检查重置密码的验证码(重置密码就是忘记密码)
     */
    override fun checkResetCode(phone: String, countryCode: String, msgCode: String) {
        model.checkResetCode(phone, countryCode, msgCode)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe { view.showLoading(true) }
                .doAfterTerminate { view.showLoading(false) }
                .subscribe({
                    if (it.code == 200) {
                        //检查重置密码的验证码后清空本地保存时间
                        YDLCacheUtils.saveResetCodeTime(phone, "")

                        view.checkResetCodeSuccess(msgCode)
                    } else {
                        view.startAnim()
                    }
                }, {
                    view.startAnim()
                    ToastUtil.toastShort(it.message)
                })
    }

    /**
     * 验证码验证码登录
     */
    override fun loginByMsgCode(phone: String, phoneCountryCode: String, msgCode: String) {

        ActionCountUtils.count(UserHelper.getUserInfo()!!.uid, BIConstants.PART_ID_LOGIN_MAIN,
                BIConstants.POSITION_LOGIN_CLICK, YdlRetrofitUtils.SERVER_API_JAVA_URL,
                YdlRetrofitUtils.SERVER_API_JAVA_URL + "user/login_sms", BIConstants.ACTION_TYPE_CODE_LOGIN)

        var param = PhoneLoginCodeParam(msgCode, phoneCountryCode, phone)
        model.loginByMsgCode(param)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe { view.showLoading(true) }
                .doAfterTerminate { view.showLoading(false) }
                .subscribe({
                    if (it.data != null) {
                        //验证码登录成功后清空本地保存时间
                        YDLCacheUtils.saveCodeTime(phone, "")

                        saveUserData(it.data)
                        
                        if (it.data.firstLogin == 1) {//第一次登录：是注册
                            view.baiduActionBury()
                            UMEventUtils.loginSuccess("VerificationCodeActivity|Reg_succ")
                        } else {
                            ActionCountUtils.count(UserHelper.getUserInfo()!!.uid, BIConstants.PART_ID_LOGIN_MAIN,
                                    BIConstants.POSITION_LOGIN_SUCCESS_CLICK, YdlRetrofitUtils.SERVER_API_JAVA_URL,
                                    YdlRetrofitUtils.SERVER_API_JAVA_URL + "user/login_sms", BIConstants.ACTION_TYPE_CODE_LOGIN)
                            UMEventUtils.loginSuccess("VerificationCodeActivity|Login_succ")
                        }
                        view.closeActivity()
                    } else {
                        view.startAnim()
                    }
                }, {
                    view.startAnim()
                    ToastUtil.toastShort(it.message)
                })
    }

    /**
     * 保存登录信息
     */
    private fun saveUserData(userResponse: UserResponse) {
        if (!TextUtils.isEmpty(userResponse.userInfo!!.phone)) {
            YDLCacheUtils.saveUserPhone(userResponse.userInfo!!.phone)
            YDLCacheUtils.saveCountryPhoneCode(userResponse.userInfo!!.country_code)
        }
        if (userResponse.uid!!.contains(".")) {
            val bd = BigDecimal(userResponse.uid)
            var uid = bd.toPlainString()
            if (uid.endsWith(".0")) {
                uid = uid.replace(".0", "")
            }
            userResponse.userInfo!!.uid = uid
            userResponse.uid = uid
        }
        LoginUtils.saveData(userResponse)
        LogHelper.getInstance().writeLogSync("登录成功")
        LoginUtils.onLogin(userResponse)
    }
}